/**
 * Copyright 2008,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * under the License.
 */
package net.sourceforge.tink.app.engine;

import net.sourceforge.tink.model.Output;
import net.sourceforge.tink.model.TinkException;
import net.sourceforge.tink.model.helpers.ProcessorBuilder;
import net.sourceforge.tink.model.helpers.PublisherBuilder;

import org.apache.tools.ant.types.FileSet;

import java.io.File;
import java.io.IOException;

import java.net.URI;

public class SiteConfigWrapper implements ProcessorBuilder.Config, PublisherBuilder.Config
{
    public static final String DEFAULT_INCLUDES = "**/*.html **/*.htm";
    public static final String TMP_PREFIX = "tink.";
    public static final String TMP_SUFFIX = ".site";
    private final SiteConfig config;
    private final SiteConfig defaults;
    private final Output output;
    private final File workDir;

    public SiteConfigWrapper(SiteConfig config, SiteConfig defaults, Output output) throws TinkException
    {
        this.config = config;
        this.defaults = (defaults == null) ? config : defaults;
        this.output = output;
        if (get(config.getStage(), defaults.getStage()) == null)
        {
            try
            {
                workDir = File.createTempFile(TMP_PREFIX, TMP_SUFFIX);
                workDir.delete();
                workDir.mkdir();
            }
            catch (IOException x)
            {
                throw new TinkException("Can't make working directory", x);
            }
        }
        else
        {
            workDir = null;
        }
    }

    @Override public File getData()
    {
        return get(config.getData(), defaults.getData());
    }

    @Override public File getDestdir()
    {
        return getWorkDir();
    }

    @Override public String getExcludes()
    {
        return get(config.getExcludes(), defaults.getExcludes());
    }

    @Override public File getExcludesfile()
    {
        return null;
    }

    @Override public String getIncludes()
    {
        return get(config.getIncludes(), defaults.getIncludes(), DEFAULT_INCLUDES);
    }

    @Override public File getIncludesfile()
    {
        return null;
    }

    @Override public File getLocal()
    {
        return getWorkDir();
    }

    @Override public Output getOutput()
    {
        return output;
    }

    @Override public String getPassword()
    {
        return get(config.getPassword(), defaults.getPassword());
    }

    @Override public URI getRemote()
    {
        return get(config.getRemote(), defaults.getRemote());
    }

    @Override public FileSet getSrc()
    {
        return null;
    }

    @Override public File getSrcdir()
    {
        return get(config.getSource(), defaults.getSource());
    }

    @Override public File getTemplate()
    {
        return get(config.getTemplate(), defaults.getTemplate());
    }

    @Override public String getUser()
    {
        return get(config.getUser(), defaults.getUser(), System.getProperty("user.name"));
    }

    public void cleanup()
    {
        if (workDir != null)
        {
            delete(workDir);
        }
    }

    protected File getWorkDir()
    {
        return (workDir == null) ? get(config.getStage(), defaults.getStage()) : workDir;
    }

    protected void delete(File file)
    {
        if (file.isDirectory())
        {
            for (File child : file.listFiles())
            {
                delete(child);
            }
        }

        file.delete();
    }

    protected <T> T get(T forceValue, T configValue)
    {
        return get(forceValue, configValue, null);
    }

    protected <T> T get(T forceValue, T configValue, T defaultValue)
    {
        return (forceValue == null) ? ((configValue == null) ? defaultValue : configValue) : forceValue;
    }
}
